//
//  PageControl.swift
//  Landmarks
//
//  Created by Nico Weber on 12/20/24.
//

import SwiftUI
import AppKit

struct ScrollControl: NSViewRepresentable {
    @Binding var pos: Double

    var child: ScrollChild

    // ^NSViewRepresentable
    func makeCoordinator() -> Coordinator {
        NSLog("make coordinator")

        return Coordinator(self)
    }

    // ^NSViewRepresentable
    func makeNSView(context: Context) -> NSScrollView {
        NSLog("make control")
        let control = NSScrollView()

        control.documentView = NSView()

        // documentView has big bounds, bigger than max texture size
        control.documentView?.setFrameSize(NSSize(width:20000, height:10000))

        control.hasHorizontalScroller = true
        control.hasVerticalScroller = true

        control.contentView.postsBoundsChangedNotifications = true

        NotificationCenter.default.addObserver(
            context.coordinator,
            selector: #selector(Coordinator.updateBounds(sender:)),
            name: NSView.boundsDidChangeNotification,
            object: control.contentView)

        return control
    }

    // ^NSViewRepresentable
    func updateNSView(_ nsView: NSScrollView, context: Context) {
        NSLog("make view")

        // XXX do i have to update this every time?
        // XXX is there a nicer way than just setting this?
        // https://stackoverflow.com/questions/77913756/nsviewrepresentable-updatensview-is-not-called-unless-the-modified-binding-varia
        context.coordinator.scrollView = nsView

        // Host a SwiftUI view as a child of the scroller
        context.coordinator.hostingController = NSHostingController(rootView: child)

//        NSLog("size \(context.coordinator.scrollView!.frame)")

        nsView.documentView?.subviews.forEach({ $0.removeFromSuperview() })
        nsView.documentView?.addSubview(context.coordinator.hostingController!.view)

        // the NSScrollView has its final size only after this :/
        // could probably create constraints instead.
        DispatchQueue.main.async {
            NSLog("size \(context.coordinator.scrollView!.frame); \(context.coordinator.hostingController!.view.frame)")
            context.coordinator.hostingController!.view.frame = NSMakeRect(
                0, 0,
                nsView.contentView.bounds.width, nsView.contentView.bounds.height)
        }

        nsView.setNeedsDisplay(nsView.bounds)

    }

    class Coordinator: NSObject {
        var control: ScrollControl
        var scrollView: NSScrollView?

        var hostingController: NSHostingController<ScrollChild>?

        init(_ control: ScrollControl) {
            self.control = control
        }

        @objc
        func updateBounds(sender: NSNotification) {
            // https://developer.apple.com/library/archive/documentation/Cocoa/Conceptual/NSScrollViewGuide/Articles/SynchroScroll.html
            guard let changedContentView: NSClipView = sender.object as? NSClipView else {
                return
            }

            let changedBoundsOrigin: NSPoint = changedContentView.documentVisibleRect.origin

//            let curOffset: NSPoint = scrollView!.contentView.bounds.origin
//            var newOffset: NSPoint = curOffset;
            
//            newOffset.x = changedBoundsOrigin.x
//            newOffset.y = changedBoundsOrigin.y

//            if (!NSEqualPoints(curOffset, changedBoundsOrigin)) {
//                NSLog("scroll to point \(newOffset)")
//            }
//            NSLog("scroll to point \(changedBoundsOrigin)")
//            control.pos = changedBoundsOrigin.x / 20000

            control.pos = changedBoundsOrigin.y / 1000

            if let hostingController {
                hostingController.view.frame = NSMakeRect(
                    changedBoundsOrigin.x, changedBoundsOrigin.y,
                    scrollView!.contentView.bounds.width, scrollView!.contentView.bounds.height)
//                NSLog("changing hosting bounds \(hostingController.view.bounds)")
            }
        }
    }
}
